/*********************************************************************
 * Copyright 2011 Bartek thindil Jasicki
 *
 *  This file is part of Steam Fort.
 *
 *  Steam Fort is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Steam Fort is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Steam Fort.  If not, see <http://www.gnu.org/licenses/>.
*********************************************************************/
#include "gui/gui.hpp"
#include "ui.hpp"
#include "main.hpp"
#include "designation.hpp"
#include <iostream>

Container *marksmenu = new Container(0, 0, 100, 75);
VBox *markmenu = new VBox(60, 2, 1);
int designtype = NONE;
int started[2];

/**
 * Show/hide designation menu
 * @param visible If true, show menu, otherwise hide.
 */
void setVisibleDesignation(bool visible)
{
  markmenu->setVisible(false);
  marksmenu->setVisible(visible);
  escinfo->setVisible(visible);
}

/**
 * Start designating things.
 * @param w Unused.
 * @param data Type of designation (trees, rocks, etc)
 */
void markClb(Widget *w, void* data)
{
  string* p = (string*)data;
  state = MARK;
  stage = NONE;
  if (p->compare("trees") == 0)
    {
      designtype = TREE;
    }
  else if (p->compare("rocks") == 0)
    {
      designtype = ROCK;
    }
  started[0] = NONE;
  started[1] = NONE;
  if (gtime[2] < 3)
    {
      Widget::setForegroundColor(TCODColor::black, TCODColor::red);
    }
  setVisibleDesignation(false);
  markmenu->setVisible(true);
  draw_fort();
}

/**
 * Build designation user interface.
 */
void buildDesignationUI()
{
  marksmenu->addWidget(new Button(37, 20, 10, 1, "Mark trees", "", markClb, new string("trees")));
  marksmenu->addWidget(new Button(37, 22, 10, 1, "Mark rocks", "", markClb, new string("rocks")));
  marksmenu->setVisible(false);
  ToolBar *dmenu = new ToolBar(60, 30, "Info");
  dmenu->addWidget(new Label(0, 0, "First left click start selection."));
  dmenu->addWidget(new Label(0, 0, "Second left click end selection."));
  dmenu->addWidget(new Label(0, 0, "Hit ESC to back to menu."));
  markmenu->addWidget(dmenu);
  markmenu->setVisible(false);
}

/**
 * Designate selected resources.
 * @param mouse Mouse information.
 */
void designation(TCOD_mouse_t *mouse)
{
  int drawx, drawy, width, height;
  if (mouse->cx > started[0])
    {
      drawx= started[0];
    }
  else
    {
      drawx = mouse->cx;
    }
  if (mouse->cy > started[1])
    {
      drawy = started[1];
    }
  else
    {
      drawy = mouse->cy;
    }
  width = abs(started[0] - mouse->cx);
  height = abs(started[1] - mouse->cy);
  if (mouse->lbutton_pressed)
    {
      if (started[0] == NONE)
	{
	  started[0] = mouse->cx;
	  started[1] = mouse->cy;
	}
      else
	{
	  for (int i = 0; i < (int)fortobjs.size(); i++)
	    {
	      if ((fortobjs[i].x < drawx + screenx) || (fortobjs[i].x > (drawx + screenx + width)) || (fortobjs[i].y < drawy + screeny) || (fortobjs[i].y > (drawy + screeny + height)))
		{
		  continue;
		}
	      if (fortobjs[i].types.contains(designtype))
		{
		  fortobjs[i].designated = true;
		  designated[designtype].push_back(i);
		}
	    }
	  if (designated[designtype].size() == 0)
	    {
	      designated[designtype].clear();
	    }
	  state = MARKS;
	  Widget::setForegroundColor(TCODColor::white, TCODColor::red);
	  setVisibleDesignation(true);
	  return;
	}
    }
  if (started[0] != NONE)
    {
      draw_fort();
      offconsole->clear();
      TCODConsole::blit(offconsole, 0, 0, width, height, TCODConsole::root, drawx, drawy, 1.0, 0.5);
    }
}
