/*********************************************************************
 * Copyright 2011 Bartek thindil Jasicki
 *
 *  This file is part of Steam Fort.
 *
 *  Steam Fort is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Steam Fort is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Steam Fort.  If not, see <http://www.gnu.org/licenses/>.
*********************************************************************/
#include "libtcod.hpp"
#include "mobile.hpp"
#include "utils.hpp"
#include "main.hpp"
#include <iostream>
#include "gui/gui.hpp"
#include "ui.hpp"
#include <stdio.h>
#include "message.hpp"
#include <map>
#include "adventure.hpp"
#include "game.hpp"

Container *mobsmenu = new Container(0, 0, SCREENWIDTH, SCREENHEIGHT);
Container *mobmenu = new Container(30, 10, 80, 70);
ToolBar *tmpbar = new ToolBar(65, 16, "Available items");
Container *mobinv = new Container(0, 0, SCREENWIDTH, SCREENHEIGHT);
ToolBar *inv = new ToolBar(2, 2, "Inventory");
ToolBar *available = new ToolBar(60, 2, "Available items");

/**
 * Standard class constructor.
 * Set mobile path finder.
 */
Mobile::Mobile()
{
  taken = NONE;
  description = "";
  path = 0;
  flying = false;
}

/**
 * Class constructor.
 * Create new mobile and fill some variables.
 * @param c Character used to display mobile on screen
 * @param col Color for character used to display mobile on screen
 * @param t Type of mobile (player, robot, etc)
 */
Mobile::Mobile(char c, TCODColor col, int t)
{
  ch = c;
  color = col;
  type = t;
  work = (char *)"IDLE";
  targettype = -1;
  target = -1;
  worktime = 0;
  carries = -1;
  taken = NONE;
  workstage = 0;
  for (int i = 0; i < 16; i++)
    {
      stats[i] = NONE;
    }
  for (int i = 0; i < 5; i++)
    {
      equipment[i] = NONE;
    }
  walktime = 0;
  description = "";
  for (int i = 0; i < 3; i++)
    {
      ais[i] = NONE;
    }
  path = 0;
  flying = false;
}

/**
 * Add mob to mobs vector.
 * @param start True mean we start game.
 */
int Mobile::addmob(bool start)
{
  int mob = NONE;
  if (state == FORT)
    {
      if ((fortmobs.size() > 0) && (!start))
	{
	  for (int i = 0; i < (int)fortmobs.size(); i++)
	    {
	      if ((fortmobs[i].x == NONE) || (fortmobs[i].y == NONE))
		{
		  fortmobs[i] = *this;
		  mob = i;
		  break;
		}
	    }
	}
      if (mob == NONE)
	{
	  mob = fortmobs.size();
	  fortmobs.push_back(*this);
	}
      fortmobs[mob].path = new TCODPath(fovmap);
    }
  else
    {
      if ((advmobs.size() > 0) && (!start))
	{
	  for (int i = 0; i < (int)advmobs.size(); i++)
	    {
	      if ((advmobs[i].x == NONE) || (advmobs[i].y == NONE))
		{
		  advmobs[i] = *this;
		  mob = i;
		  break;
		}
	    }
	}
      if (mob == NONE)
	{
	  mob = advmobs.size();
	  advmobs.push_back(*this);
	}
      advmobs[mob].path = new TCODPath(fovmap);
    }
  return mob;
}

/**
 * Draw mobile on sceen.
 * @param x1 X-axis position on screen
 * @param y1 Y-axis position on screen
 * @param mindex Index of mobile.
 * @param start True means we start game. Default is false.
 */
void Mobile::setmob(int x1, int y1, int mindex, bool start)
{
  TCODRandom *generator = TCODRandom::getInstance();
  while (true)
    {
      if (valid_pos(x1, y1))
	{
	  x = x1;
	  y = y1;
	  if (mindex == NONE)
	    {
	      mindex = addmob(start);
	    }
	  update_map_mob(x, y, mindex);
	  break;
	}
      else
	{
	  x1 = generator->getInt(x1 - 5, x1 + 5);
	  if ((x1 < 0) || (x1 >= MAPWIDTH))
	    {
	      x1 = MAPWIDTH / 2;
	    }
	  y1 = generator->getInt(y1 - 5, y1 + 5);
	  if ((y1 < 0) || (y1 >= MAPHEIGHT))
	    {
	      y1 = MAPHEIGHT / 2;
	    }
	}
    };
}

/**
 * Generate stats of mobile.
 * @param newlevel New level of mobile. Default -1.
 */
void Mobile::generatestats(int newlevel)
{
  if (newlevel != NONE)
    {
      stats[10] = newlevel;
    }
  TCODRandom *generator = TCODRandom::getInstance();
  char *c;
  for (int i = 2; i <= 6; i++)
    {
      c = new char[4];
      sprintf(c, "1d%d + %d", stats[i], stats[10]);
      stats[i] = generator->diceRoll(c);
    }
  c = new char[4];
  sprintf(c, "1d%d + %d", stats[9], stats[10]);
  stats[9] = generator->diceRoll(c);
  stats[7] = stats[9] + stats[7];
  stats[8] = stats[9] + stats[8];
  stats[14] = stats[14] + stats[6];
  stats[15] = stats[15] + stats[6];
}

/**
 * Generate name for mob.
 */
void Mobile::generatename()
{
  switch (type)
    {
    case PLAYER:
      TCODNamegen::parse("data/namegen/mingos_standard.cfg");
      name = TCODNamegen::generate((char *)"male");
      break;
    default:
      TCODNamegen::parse("data/namegen/jice_mesopotamian.cfg");
      name = TCODNamegen::generate((char *)"Mesopotamian male");
      break;
    }
  TCODNamegen::destroy();
}

/**
 * Generate path to selected target.
 * @param x1 X-axis position of target
 * @param y1 Y-axis position of target
 * @param range Size of range in which destination will be checked.
 * @return True if path was computed, otherwise false
 */
bool Mobile::generatepath(int x1, int y1, int range)
{
  if ((x1 < 0) || (y1 < 0))
    {
      return false;
    }
  if (!valid_pos(x1, y1))
    {
      bool walkable = true;
      if (state == FORT)
	{
	  walkable = !flying;
	}
      TCODList<int> steps;
      for (int y2 = 0 - range; y2 <= range; y2++)
	{
	  for (int x2 = 0 - range; x2 <= range; x2++)
	    {
	      if (valid_pos(x1 + x2, y1 + y2, walkable))
		{
		  path->compute(x, y, x1 + x2, y1 + y2);
		  steps.push(path->size());
		}
	    }
	}
      int min = 1000, index;
      for (int i = 0; i < steps.size(); i++)
	{
	  if (min > steps.get(i))
	    {
	      min = steps.get(i);
	      index = i;
	    }
	}
      int index2 = 0;
      for (int y2 = 0 - range; y2 <= range; y2++)
	{
	  for (int x2 = 0 - range; x2 <= range; x2++)
	    {
	      if (valid_pos(x1 + x2, y1 + y2, walkable))
		{
		  if (index2 == index)
		    {
		      path->compute(x, y, x1 + x2, y1 + y2);
		      return true;
		    }
		  index2++;
		}
	    }
	}
    }
  else
    {
      path->compute(x, y, x1, y1);
      return true;
    }
  return false;
}

/**
 * Move mobile by one step in path.
 * @param time Length in seconds of the last rendered frame
 * @param mindex Index of current mob.
 */
void Mobile::movestep(float time, int mindex)
{
  if (!path->isEmpty())
    {
      if ((stats[0] == 0) && (state == FORT))
	{
	  return;
	}
      walktime -= time;
      if (walktime <= 0)
	{
	  if (state == FORT)
	    {
	      stats[0]--;
	    }
	  update_map_mob(x, y, NONE);
	  path->walk(&x, &y, true);
	  fovmap->setProperties(x, y, true, false);
	  if (targettype == MOB)
	    {
	      int x1, y1, index, range = 1;
	      path->getDestination(&x1, &y1);
	      if (state == FORT)
		{
		  index = find_order(work);
		  if ((fortorders[index].type == COMBAT) && (equipment[4] != NONE))
		    {
		      int windex = equipment[4];
		      range = fortobjs[windex].range;
		    }
		  if (index == NONE)
		    {
		      index = find_spell(work);
		      if (index != NONE)
			{
			  range = protospells[index].range;
			}
		    }
		  if ((abs(x1 - fortmobs[target].x) > range) || (abs(y1 - fortmobs[target].y) > range))
		    {
		      if (!generatepath(fortmobs[target].x, fortmobs[target].y, range))
			{
			  generatepath(x, y);
			}
		    }
		}
	      else
		{
		  if (equipment[4] != NONE)
		    {
		      int windex = equipment[4];
		      range = advobjs[windex].range;
		    }
		  index = find_spell(work);
		  if (index != NONE)
		    {
		      range = protospells[index].range;
		    }
		  if ((abs(x1 - advmobs[target].x) > range) || (abs(y1 - advmobs[target].y) > range))
		    {
		      if (!generatepath(advmobs[target].x, advmobs[target].y, range))
			{
			  generatepath(x, y);
			}
		    }
		}
	    }
	  update_map_mob(x, y, mindex);
	  if (state == FORT)
	    {
	      walktime = 1 - ((float)stats[5] / 100);
	    }
	  else
	    {
	      walktime = 0.5 - ((float)stats[5] / 200);
	    }
	  if ((stats[0] <= 0) && (type == ROBOT))
	    {
	      addMessage(new Message("Steambot need refueling.", x, y));
	    }
	}
    }
  else if (targettype == MOB)
    {
      int x1, y1, index, range = 1;
      path->getDestination(&x1, &y1);
      if (state == FORT)
	{
	  index = find_order(work);
	  if ((fortorders[index].type == COMBAT) && (equipment[4] != NONE))
	    {
	      int windex = equipment[4];
	      range = fortobjs[windex].range;
	    }
	  if (index == NONE)
	    {
	      index = find_spell(work);
	      if (index != NONE)
		{
		  range = protospells[index].range;
		}
	    }
	  if ((abs(x1 - fortmobs[target].x) > range) || (abs(y1 - fortmobs[target].y) > range))
	    {
	      if (!generatepath(fortmobs[target].x, fortmobs[target].y))
		{
		  generatepath(x, y);
		}
	    }
	}
      else
	{
	  if (equipment[4] != NONE)
	    {
	      int windex = equipment[4];
	      range = advobjs[windex].range;
	    }
	  index = find_spell(work);
	  if (index != NONE)
	    {
	      range = protospells[index].range;
	    }
	  if ((abs(x1 - advmobs[target].x) > range) || (abs(y1 - advmobs[target].y) > range))
	    {
	      if (!generatepath(advmobs[target].x, advmobs[target].y))
		{
		  generatepath(x, y);
		}
	    }
	}
    }
}

/**
 * Gaining experience by mob.
 * @param amount Amount of experience points gained.
 */
void Mobile::gainexp(int amount)
{
  if (type != PLAYER)
    {
      return;
    }
  stats[11] += amount;
  int needexp = stats[10] * stats[12];
  if (stats[11] >= needexp)
    {
      stats[10]++;
      stats[11] -= needexp;
      stats[13] += 3;
    }
}

/**
 * Set working time of mob
 * @param time New value of time or length in seconds of the last rendered frame
 * @param stat Index of stat to use. Default -1.
 */
void Mobile::setworktime(float time, int stat)
{
  switch (stat)
    {
    case -2:
      worktime -= time;
      break;
    case -1:
      worktime = time;
      break;
    default:
      worktime = time - ((stats[stat] / 100) * time);
      break;
    }
}

/**
 * Remove mobile from game.
 */
void Mobile::deletemob()
{
  //Drop equipment
  int index;
  for (int i = 0; i < 5; i++)
    {
      index = equipment[i];
      if (index == NONE)
	{
	  continue;
	}
      if (state == FORT)
	{
	  fortobjs[index].setitem(x, y, index);
	}
      else
	{
	  advobjs[index].setitem(x, y, index);
	}
    }
  //Drop inventory
  for (int i = 0; i < inventory.size(); i++)
    {
      index = inventory.get(i);
      if (state == FORT)
	{
	  fortobjs[index].setitem(x, y, index);
	}
      else
	{
	  advobjs[index].setitem(x, y, index);
	}
    }
  //Drop loot
  int chance = TCODRandom::getInstance()->getInt(0, 100);
  if ((chance < 50) && (!loot.isEmpty()))
    {
      TCODList<int> *tmptypes = new TCODList<int>();
      for (int i = 0; i < loot.size(); i++)
	{
	  tmptypes->push(loot.get(i));
	  index = find_proto_obj("", tmptypes);
	  protoobjs[index].setitem(x, y);
	  tmptypes->clear();
	}
      delete tmptypes;
    }
  inventory.clear();
  delete path;
  x = NONE;
  y = NONE;
}

/**
 * Build crew user interface.
 */
void buildMobileUI()
{
  mobsmenu->setVisible(false);
  mobmenu->setVisible(false);
  tmpbar->setVisible(false);
  mobinv->addWidget(available);
  mobinv->addWidget(inv);
  mobinv->setVisible(false);
}

/**
 * Update user stats
 * @param w Unused
 * @param data Value from button from UI.
 */
void advanceClb(Widget *w, void* data)
{
  int *index = static_cast<int*>(data);
  if (state == MOBINFO)
    {
      if (fortmobs[stage].stats[*index] == 50)
	{
	  return;
	}
      fortmobs[stage].stats[*index]++;
      fortmobs[stage].stats[13]--;
      if (*index == 9)
	{
	  fortmobs[stage].stats[8]++;
	  fortmobs[stage].stats[7]++;
	}
      else if (*index == 6)
	{
	  fortmobs[stage].stats[14]++;
	  fortmobs[stage].stats[15]++;
	}
    }
  else
    {
      if (advmobs[stage].stats[*index] == 50)
	{
	  return;
	}
      advmobs[stage].stats[*index]++;
      advmobs[stage].stats[13]--;
      if (*index == 9)
	{
	  advmobs[stage].stats[8]++;
	  advmobs[stage].stats[7]++;
	}
      else if (*index == 6)
	{
	  advmobs[stage].stats[14]++;
	  advmobs[stage].stats[15]++;
	}
    }
  showMobInfo(stage);
}

/**
 * Wear selected item
 * @param w Unused.
 * @param data Location and name of item to wear.
 */
void equip2Clb(Widget *w, void *data)
{
  string* p = (string*)data;
  size_t pos = p->find(';');
  string loc = p->substr(0, pos);
  pos++;
  string name = p->substr(pos);
  string locs[] = {"head", "arms", "torso", "legs", "weapon"};
  int index;
  for (int i = 0; i < 5; i++)
    {
      if (loc.compare(locs[i]) == 0)
	{
	  index = i;
	  break;
	}
    }
  int index2 = NONE;
  if (state == MOBINFO)
    {
      int index3, x, y;
      index2 = find_obj(name);
      //Remove old equipment
      if (fortmobs[stage].equipment[index] != NONE)
	{
	  index3 = fortmobs[stage].equipment[index];
	  fortobjs[index3].setitem(fortmobs[stage].x, fortmobs[stage].y, index3);
	}
      fortmobs[stage].equipment[index] = index2;
      //Put new equipment
      if (index2 != NONE)
	{
	  x = fortobjs[index2].x;
	  y = fortobjs[index2].y;
	  update_map_item(x, y, index2, true);
	  fortobjs[index2].useitem();
	}
    }
  else
    {
      for (int i = 0; i < (int)advobjs.size(); i++)
	{
	  if (advobjs[i].name.compare(name) == 0)
	    {
	      index2 = i;
	      break;
	    }
	}
      if (advmobs[stage].equipment[index] != NONE)
	{
	  advmobs[stage].inventory.push(advmobs[stage].equipment[index]);
	}
      advmobs[stage].equipment[index] = index2;
    }
  tmpbar->setVisible(false);
  showMobInfo(stage);
}

/**
 * Show available equipment for selected location
 * @param w Unused.
 * @param data Name of location to show.
 */
void equipClb(Widget *w, void *data)
{
  string* p = (string*)data;
  tmpbar->clear();
  if (!tmpbar->isVisible())
    {
      tmpbar->setVisible(true);
    }
  tmpbar->addWidget(new Button("None", NULL, equip2Clb, new string(*p + ";NONE")));
  TCODList<int> tmptypes;
  if (fortmobs[stage].type == PLAYER)
    {
      if (p->compare("head") == 0)
	{
	  tmptypes.push(HELMET);
	}
      else if (p->compare("arms") == 0)
	{
	  tmptypes.push(BRACERS);
	}
      else if (p->compare("torso") == 0)
	{
	  tmptypes.push(ARMOR);
	}
      else if (p->compare("legs") == 0)
	{
	  tmptypes.push(GREAVES);
	}
    }
  else
    {
      if (p->compare("head") == 0)
	{
	  tmptypes.push(HUPGRADE);
	}
      else if (p->compare("arms") == 0)
	{
	  tmptypes.push(AUPGRADE);
	}
      else if (p->compare("torso") == 0)
	{
	  tmptypes.push(TUPGRADE);
	}
      else if (p->compare("legs") == 0)
	{
	  tmptypes.push(LUPGRADE);
	}
    }
  if (p->compare("weapon") == 0)
    {
      tmptypes.push(WEAPON);
    }
  if (state == MOBINFO)
    {
      TCODList<int> result = find_all_obj(tmptypes);
      if (result.size() > 0)
	{
	  for (int i = 0; i < result.size(); i++)
	    {
	      tmpbar->addWidget(new Button(fortobjs[result.get(i)].name.c_str(), NULL, equip2Clb, new string(*p + ";" + fortobjs[result.get(i)].name)));
	    }
	}
    }
  else
    {
      if (!advmobs[stage].inventory.isEmpty())
	{
	  int index;
	  for (int i = 0; i < advmobs[stage].inventory.size(); i++)
	    {
	      index = advmobs[stage].inventory.get(i);
	      tmpbar->addWidget(new Button(advobjs[index].name.c_str(), NULL, equip2Clb, new string(*p + ";" + advobjs[index].name)));
	    }
	}
    }
  tmpbar->computeSize();
  showMobInfo(stage);
}

/**
 * Rename selected mob
 * @param wid Unused.
 * @param val New name of mob.
 * @param data Index of mob.
 */
void renameClb(Widget *wid, char *val, void *data)
{
  int *index = static_cast<int*>(data);
  string *value = new string(val);
  if (value->size() == 0)
    {
      return;
    }
  fortmobs[*index].name = *value;
}

/**
 * Add item to inventory
 * @param w Unused.
 * @param data Name of item to add to inventory.
 */
void addinvClb(Widget *w, void *data)
{
  if (fortmobs[stage].inventory.size() == 20)
    {
      return;
    }
  string *name = static_cast<string*>(data);
  int index = find_obj(*name);
  int x = fortobjs[index].x;
  int y = fortobjs[index].y;
  update_map_item(x, y, index, true);
  fortobjs[index].stored = false;
  fortobjs[index].useitem();
  fortmobs[stage].inventory.push(index);
  drawItems(stage);
}

/**
 * Remove item from inventory
 * @param w Unused
 * @param data Index of item to remove
 */
void removeinvClb(Widget *w, void *data)
{
  int *index = static_cast<int*>(data);
  if (state == MOBINV)
    {
      fortobjs[*index].setitem(fortmobs[stage].x, fortmobs[stage].y, *index);
      fortmobs[stage].inventory.remove(*index);
    }
  else
    {
      advobjs[*index].setitem(advmobs[stage].x, advmobs[stage].y, *index);
      advmobs[stage].inventory.remove(*index);
    }
  drawItems(stage);
}

/**
 * Draw all items which can be put to inventory.
 * @param index Index of mob.
 */
void drawItems(int index)
{
  mobmenu->setVisible(false);
  mobinv->setVisible(true);
  inv->clear();
  inv->addWidget(new Label(0, 0, ""));
  int line = 4;
  if (state == MOBINV)
    {
      available->setVisible(true);
      available->clear();
      available->addWidget(new Label(0, 0, ""));
      map<string, int> items;
      for (int i = 0; i < (int)fortobjs.size(); i++)
	{
	  if ((fortobjs[i].x > -1) && (fortobjs[i].y > -1) && (fortobjs[i].types.contains(OBJECT)) && (fortobjs[i].stored))
	    {
	      if (items.find(fortobjs[i].name) == items.end())
		{
		  items[fortobjs[i].name] =  0;
		}
	      items[fortobjs[i].name] ++;
	    }
	}
      //Show all available items
      char* c;
      if (!items.empty())
	{
	  for (map<string, int>::iterator i = items.begin(); i != items.end(); i++)
	    {
	      c = new char[30];
	      sprintf(c, "%s: %d", (*i).first.c_str(), (*i).second);
	      available->addWidget(new Button(c, "Add item to inventory", addinvClb, new string((*i).first)));
	      line ++;
	    }
	}
      available->addWidget(new Label(0, line, ""));
      available->computeSize();
    }
  else
    {
      available->setVisible(false);
    }
  //Show player inventory
  int index2;
  line = 4;
  if (state == MOBINV)
    {
      if (!fortmobs[index].inventory.isEmpty())
	{
	  for (int i = 0; i < fortmobs[index].inventory.size(); i++)
	    {
	      index2 = fortmobs[index].inventory.get(i);
	      inv->addWidget(new Button(fortobjs[index2].name.c_str(), "Remove item from inventory", removeinvClb, new int(index2)));
	      line++;
	    }
	}
    }
  else
    {
      if (!advmobs[index].inventory.isEmpty())
	{
	  for (int i = 0; i < advmobs[index].inventory.size(); i++)
	    {
	      index2 = advmobs[index].inventory.get(i);
	      inv->addWidget(new Button(advobjs[index2].name.c_str(), "Remove item from inventory", removeinvClb, new int(index2)));
	      line++;
	    }
	}
    }
  inv->addWidget(new Label(0, line, ""));
  inv->computeSize();
  mobinv->addWidget(inv);
}

/**
 * Manage player inventory
 * @param w Unused.
 * @param data Index of player.
 */
void inventoryClb(Widget *w, void *data)
{
  int *index = static_cast<int*>(data);
  stage = *index;
  if (state == MOBINFO)
    {
      state = MOBINV;
    }
  else
    {
      state = MOBINV2;
    }
  drawItems(*index);
}

/**
 * Show informations about selected mobile.
 * @param index Index of selected mobile.
 * @return Index of selected mobile.
 */
int showMobInfo(int index)
{
  if (index == NONE)
    {
      mobmenu->setVisible(false);
      mobinv->setVisible(false);
      return NONE;
    }
  mobinv->setVisible(false);
  mobsmenu->setVisible(false);
  int line = 18;
  mobmenu->clear();
  mobmenu->setVisible(true);
  Mobile *mob = new Mobile();
  if (state == MOBINFO)
    {
      mob = &fortmobs[index];
    }
  else
    {
      mob = &advmobs[index];
    }
  //Show mob stats
  TextBox *txtbox = new TextBox(10, 16, 15, 20, "Name: ", mob->name.c_str(), NULL);
  txtbox->setCallback(renameClb, new int(index));
  mobmenu->addWidget(txtbox);
  char* c;
  if (mob->type == PLAYER)
    {
      c = new char[10];
      mobmenu->addWidget(new Label(10, line, "Level:"));
      sprintf(c, "%d", mob->stats[10]);
      mobmenu->addWidget(new Label(22, line, c));
      c = new char[10];
      sprintf(c, "%d", mob->stats[11]);
      mobmenu->addWidget(new Label(10, line + 2, "Exp:"));
      mobmenu->addWidget(new Label(22, line + 2, c));
      mobmenu->addWidget(new Label(10, line + 4, "Points:"));
      c = new char[10];
      sprintf(c, "%d", mob->stats[13]);
      mobmenu->addWidget(new Label(22, line + 4, c));
      line = 24;
    }
  c = new char[10];
  int value = mob->stats[0];
  if (value < 0)
    {
      value = 0;
    }
  sprintf(c, "%d/%d", value, mob->stats[1]);
  mobmenu->addWidget(new Label(10, line, "Energy:"));
  mobmenu->addWidget(new Label(22, line, c));
  c = new char[10];
  sprintf(c, "%d/%d", mob->stats[7], mob->stats[8]);
  mobmenu->addWidget(new Label(10, line + 2, "Hit Points:"));
  mobmenu->addWidget(new Label(22, line + 2, c));
  if (mob->type == PLAYER)
    {
      mobmenu->addWidget(new Label(10, line + 4, "Mana Points:"));
      c = new char[10];
      sprintf(c, "%d/%d", mob->stats[14], mob->stats[15]);
      mobmenu->addWidget(new Label(22, line + 4, c));
      line = 26;
    }
  mobmenu->addWidget(new Label(10, line + 4, "Order:"));
  mobmenu->addWidget(new Label(22, line + 4, mob->work.c_str()));
  if ((mob->type == PLAYER) && (mob->stats[13] > 0))
    {
      mobmenu->addWidget(new Button(7, 32, 2, 1, "+", NULL, advanceClb, new int(2)));
      mobmenu->addWidget(new Button(7, 34, 2, 1, "+", NULL, advanceClb, new int(3)));
      mobmenu->addWidget(new Button(7, 36, 2, 1, "+", NULL, advanceClb, new int(4)));
      mobmenu->addWidget(new Button(7, 38, 2, 1, "+", NULL, advanceClb, new int(5)));
      mobmenu->addWidget(new Button(7, 40, 2, 1, "+", NULL, advanceClb, new int(6)));
      mobmenu->addWidget(new Button(7, 42, 2, 1, "+", NULL, advanceClb, new int(9)));
    }
  c = new char[4];
  sprintf(c, "%d", mob->stats[2]);
  mobmenu->addWidget(new Label(22, 32, c));
  c = new char[4];
  sprintf(c, "%d", mob->stats[3]);
  mobmenu->addWidget(new Label(22, 34, c));
  c = new char[4];
  sprintf(c, "%d", mob->stats[4]);
  mobmenu->addWidget(new Label(22, 36, c));
  c = new char[4];
  sprintf(c, "%d", mob->stats[5]);
  mobmenu->addWidget(new Label(22, 38, c));
  c = new char[4];
  sprintf(c, "%d", mob->stats[6]);
  mobmenu->addWidget(new Label(22, 40, c));
  mobmenu->addWidget(new Label(10, 32, "Strenght:"));
  mobmenu->addWidget(new Label(10, 34, "Dexterity:"));
  mobmenu->addWidget(new Label(10, 36, "Senses:"));
  mobmenu->addWidget(new Label(10, 38, "Speed:"));
  mobmenu->addWidget(new Label(10, 40, "Memory:"));
  if (mob->type == PLAYER)
    {
      mobmenu->addWidget(new Label(10, 42, "Condition:"));
      c = new char[4];
      sprintf(c, "%d", mob->stats[9]);
      mobmenu->addWidget(new Label(22, 42, c));
    }
  //Show mob equipment/upgrades
  mobmenu->addWidget(new Label(35, 16, "Head:"));
  mobmenu->addWidget(new Label(35, 18, "Arms:"));
  mobmenu->addWidget(new Label(35, 20, "Torso:"));
  mobmenu->addWidget(new Label(35, 22, "Legs:"));
  mobmenu->addWidget(new Label(35, 24, "Weapon:"));
  if (mob->type == PLAYER)
    {
      mobmenu->addWidget(new Button(35, 28, 10, 1, "Inventory", "Show player inventory", inventoryClb, new int(index)));
    }
  line = 16;
  string names[] = {"head", "arms", "torso", "legs", "weapon"};
  int index2;
  for (int i = 0; i < 5; i++)
    {
      index2 = mob->equipment[i];
      if (index2 == NONE)
	{
	  mobmenu->addWidget(new Button(48, line, 10, 1, "None", "Assign item", equipClb, new string(names[i])));
	}
      else
	{
	  if (state == MOBINFO)
	    {
	      mobmenu->addWidget(new Button(48, line, 10, 1, fortobjs[index2].name.c_str(), "Change item", equipClb, new string(names[i])));
	    }
	  else
	    {
	      mobmenu->addWidget(new Button(48, line, 10, 1, advobjs[index2].name.c_str(), "Change item", equipClb, new string(names[i])));
	    }
	}
      line += 2;
    }
  return index;
}

/**
 * Show information about selected mobile.
 * @param w Unused.
 * @param data Selected mob name.
 */
void mobClb(Widget *w, void* data)
{
  int *index = static_cast<int*>(data);
  if (state == CREW)
    {
      state = MOBINFO;
    }
  else
    {
      setVisibleAdvmenu(false);
      state = MOBINFO2;
    }
  stage = showMobInfo(*index);
}

/**
 * Show/hide information about fort crew.
 * @param visible If true, show fort crew, otherwise hide menu.
 */
void setVisibleCrew(bool visible)
{
  if (visible)
    { 
      mobmenu->setVisible(false);
      mobsmenu->clear();
      mobsmenu->addWidget(new Label(7, 2, "Name"));
      mobsmenu->addWidget(new Label(30, 2, "Type"));
      mobsmenu->addWidget(new Label(45, 2, "Energy"));
      mobsmenu->addWidget(new Label(57, 2, "Hit Points"));
      mobsmenu->addWidget(new Label(70, 2, "Order"));
      int line = 4, j = 0, value;
      char* c;
      char* c2;
      for (int i = 0; i < (int)fortmobs.size(); i++)
	{
	  c = new char[10];
	  c2 = new char[10];
	  if ((fortmobs[i].type != PLAYER) && (fortmobs[i].type != ROBOT))
	    {
	      continue;
	    }
	  mobsmenu->addWidget(new Button(7, line + j, 10, 1, fortmobs[i].name.c_str(), NULL, mobClb, new int(i)));
	  if (fortmobs[i].type == PLAYER)
	    {
	      mobsmenu->addWidget(new Label(30, line + j, "YOU"));
	    }
	  else
	    {
	      mobsmenu->addWidget(new Label(30, line + j, "Steambot"));
	    }
	  value = fortmobs[i].stats[0];
	  if (value < 0)
	    {
	      value = 0;
	    }
	  sprintf(c, "%d/%d", value, fortmobs[i].stats[1]);
	  mobsmenu->addWidget(new Label(45, line + j, c));
	  sprintf(c2, "%d/%d", fortmobs[i].stats[7], fortmobs[i].stats[8]);
	  mobsmenu->addWidget(new Label(57, line + j, c2));
	  mobsmenu->addWidget(new Label(70, line + j, fortmobs[i].work.c_str()));
	  j++;
	}
    }
  mobsmenu->setVisible(visible);
  escinfo->setVisible(visible);
  tmpbar->setVisible(false);
}

/**
 * Find proto mobile by name or type
 * @param name Name of object to find
 * @param type Type of object to find
 * @return Object index in protoobjs
 */
int find_proto_mob(std::string name, int type)
{
  int mob = -1;
  for (int i = 0; i < (int)protomobs.size(); i++)
    {
      if ((name.size() == 0) && (type != NONE))
	{
	  if (protomobs[i].type == type)
	    {
	      mob = i;
	      break;
	    }
	}
      if ((name.size() > 0) && (type == NONE))
	{
	  if (protomobs[i].name.compare(name) == 0)
	    {
	      mob = i;
	      break;
	    }
	}
      if ((name.size() > 0) && (type != NONE))
	{
	  if ((protomobs[i].name.compare(name) == 0) && (protomobs[i].type == type))
	    {
	      mob = i;
	      break;
	    }
	}
    }
  return mob;
}

/**
 * Find mobile by name or position.
 * @param name Name of mobile to find. Default is empty.
 * @param x X-axis position of mob. Default is -1.
 * @param y Y-axis position of mob. Default is -1.
 * @return Index of mobile in fortmobiles vector.
 * If not found, return -1.
 */
int find_mob(string name, int x, int y)
{
  int mob = -1;
  //Search by position
  if ((x != NONE) && (y != NONE))
    {
      mob = fortmap[y][x].mob;
      if (mob < 0)
	{
	  return NONE;
	}
      if ((name.size() > 0) && (name.compare(fortmobs[mob].name)))
	{
	  return NONE;
	}
      return mob;
    }
  for (int i = 0; i < (int)fortmobs.size(); i++)
    {
      //Search by name
      if ((name.size() > 0) && (name.compare(fortmobs[i].name)))
	{
	  continue;
	}
      mob = i;
      break;
    }
  return mob;
}

/**
 * Find all prototypes with selected type.
 * @param type Mobile type to find.
 * @param notlocation From which location mobiles should be avoid.
 * @param maxlevel Max level of mobs to find.
 * @return List of indexes of mobiles.
 */
TCODList<int> find_all_proto_mobs(int type, int notlocation, int maxlevel)
{
  TCODList<int> results;
  for (int i = 0; i < (int)protomobs.size(); i++)
    {
      //Search by type
      if (protomobs[i].type != type)
	{
	  continue;
	}
      //Search by location
      if (protomobs[i].location == notlocation)
	{
	  continue;
	}
      //Search by level
      if ((maxlevel != NONE) && (protomobs[i].stats[10] > maxlevel))
	{
	  continue;
	}
      results.push(i);
    }
  return results;
}

/**
 * Find nearest mobile from selected position
 * @param name Name of mob to find.
 * @param x X-axis of start search position.
 * @param y Y-axis of start search position.
 * @param radius Size of radius to search.
 * @param types Types of mob to find. Default empty.
 * @param fovradius Size of radius of field of view.
 * @param visible Search only for visible mobs. Default is true.
 * @return Index of mobile in fortmobs vector.
 * If mobile not found, return -1.
 */
int find_near_mob(std::string name, int x, int y, int radius, TCODList<int> *types, int fovradius, bool visible)
{
  int mob = NONE, index, sum, oldsum = 1000;
  if (visible)
    {
      fovmap->computeFov(x, y, fovradius);
    }
  for (int y1 = y - (radius + 1); y1 < y + (radius + 1); y1++)
    {
      for (int x1 = x - (radius + 1); x1 < x + (radius + 1); x1++)
	{
	  if ((y1 < 0) || (x1 < 0) || (y1 >= MAPHEIGHT) || (x1 >= MAPWIDTH))
	    {
	      continue;
	    }
	  sum = abs(x - x1) + abs(y - y1);
	  if (sum > oldsum)
	    {
	      continue;
	    }
	  if (state == FORT)
	    {
	      if (fortmap[y1][x1].mob == NONE)
		{
		  continue;
		}
	      index = fortmap[y1][x1].mob;
	      if ((name.size() > 0) && (name.compare(fortmobs[index].name)))
		{
		  continue;
		}
	      if ((!types->isEmpty()) && (!types->contains(fortmobs[index].type)))
		{
		  continue;
		}
	    }
	  else
	    {
	      if (advmap[y1][x1].mob == NONE)
		{
		  continue;
		}
	      index = advmap[y1][x1].mob;
	      if ((name.size() > 0) && (name.compare(advmobs[index].name)))
		{
		  continue;
		}
	      if ((!types->isEmpty()) && (!types->contains(advmobs[index].type)))
		{
		  continue;
		}
	    }
	  if ((visible) && (!fovmap->isInFov(x1, y1)))
	    {
	      continue;
	    }
	  mob = index;
	  oldsum  = sum;
	}
    }
  return mob;
}

/**
 * Find all mobiles with selected type
 * @param type Type of mobiles to find.
 * @return List of indexes of mobiles.
 */
TCODList<int> find_all_mobs(int type)
{
  TCODList<int> results;
  for (int i = 0; i < (int)fortmobs.size(); i++)
    {
      //Search by type
      if (fortmobs[i].type != type)
	{
	  continue;
	}
      results.push(i);
    }
  return results;
}
